from __future__ import absolute_import
from ufl.algorithms import MultiFunction
from dune.perftool.ufl.flatoperators import get_operands, construct_binary_operator
from dune.perftool.ufl.transformations import ufl_transformation

import ufl.classes as uc


class IndexPushDown(MultiFunction):
    def expr(self, o):
        return self.reuse_if_untouched(o, *tuple(self(op) for op in o.ufl_operands))

    def indexed(self, o):
        expr, idx = o.ufl_operands
        if isinstance(expr, uc.Sum):
            terms = [uc.Indexed(self(term), idx) for term in get_operands(expr)]
            return construct_binary_operator(terms, uc.Sum)
        else:
            # This is a normal indexed, we treat it as any other.
            return self.expr(o)


@ufl_transformation(name="index_pushdown")
def pushdown_indexed(e):
    """
    Removes the following antipattern from UFL expressions:
    (a+b)[i] -> a[i] + b[i]
    If similar antipatterns arise with a node other than sum,
    add the corresponding handlers here.
    """
    return IndexPushDown()(e)
